* Parsing toolkit PWITTE 1992 V0.01
*
        section code

        xdef imparse
*
imp_err_bp
        moveq #-15,d0
imp_err_ret
        rts
*
imparse
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* error% = IMPARSE(statement$,import_line$) where
*          error% can be bp, xp, bl(quote error), or ef (end of import line)
*          statement$ will contain the next statement in the import line after
*            parsing and
*          import_line$ is one line (record/column etc) of an import file,
*            truncated by one statement after parsing
*
* By repeatedly calling IMPARSE statements are extracted one by one from the
* import line until error = -10 (or LEN(import_line$)=0). Numerical data is not
* tested for conformity to QDOS standards and is treated as a string on return.
*
        move.w 0(a6,a3.l),d0    ;get first parameter type
        addq.l #8,a3            ;point to next parameter
        add.w 0(a6,a3.l),d0     ;add type
        andi.b #$0f,d0          ;remove other information
        cmpi.b #2,d0            ;2 * type 1 should be 2
        bne.s imp_err_bp        ;anything else is an error
        lea.l 8(a3),a4
        cmp.l a4,a5             ;only two parameters allowed
        bne.s imp_err_bp
        move.w $116,a2          ;get last parameter
        jsr (a2)
        bne.s imp_err_ret       ;couldn't
        move.w 0(a6,a1.l),d6    ;d6=len(import_line$)
        beq imp_eof             ;return end_of_file
        moveq #1,d7             ;2 bytes for length -2 for integer return
        add.w d6,d7             ;keep copy of total string length
*
* set up pointers and flags
*
        moveq #0,d5             ;set state (st) to 0 :bit 2=number, 1=string
        moveq #0,d4             ;set quote (q) to 0 :1=on, 0=off
        moveq #0,d2             ;statement string length
        moveq #0,d1             ;no errors
        movea.l a1,a0           ;set initial pointer
        addq.l #2,a0            ;a0 = line pointer
        movea.l a0,a2           ;a2 = statement pointer
        subq.w #1,d6            ;d6=counter
*
* parse import line
*
imp_parse
        move.b 0(a6,a0.l),d0
        cmpi.b #'"',d0
        beq.s imp_quote
        cmpi.b #',',d0
        beq.s imp_comma
        cmpi.b #13,d0
        beq.s imp_comma
        cmpi.b #'-',d0
        beq.s imp_number
        cmpi.b #'.',d0
        beq.s imp_number
        cmpi.b #'$',d0          ; V0.02
        beq.s imp_number
        cmpi.b #'0',d0
        blt.s imp_addlet
        cmpi.b #'9',d0
        bgt.s imp_addlet
imp_number
        tst.b d5                ;is statement flag set?
        bne.s imp_addlet        ; yes, add number to statement
        moveq #2,d5             ; no, start of statement type number
imp_addlet
        tst.b d5                ;if not part of a statement then error
        beq.s imp_xp
        tst.b d4                ;is quote flag on?
        bne.s imp_xp            ; yes, error
        move.b d0,0(a6,a2.l)    ;put letter into statement string
        addq.l #1,a2            ;increment pointer
        addq.w #1,d2            ;increment string length
imp_endloop
        addq.l #1,a0            ;point to next input character
        dbra d6,imp_parse       ;next loop
        add.b d5,d4             ;errors: (ss/qq) 00/01 00/00 10/01 01/00
        subq.b #2,d4            ; 10/00 is ok
        bne.s imp_xp
        moveq #0,d6             ;remaining input line length is zero
        bra.s imp_exit          ;end of input line
imp_quote
        btst #0,d5              ;string statement?
        beq.s imp_newstring     ; no
        bchg #0,d4              ; yes: toggle quote flag
        beq.s imp_endloop       ;if quote flag clear then set it and get next
        bra.s imp_addlet        ;quote flag was set so this quote is litteral
imp_newstring
        tst.b d5                ;if statement flag set then
        bne.s imp_xp            ; it must be a number - which is an error
        moveq #1,d5             ;else it is the start of a string statement
        bra.s imp_endloop       ;ignore this quote and get next character
imp_comma
        btst #0,d5              ;in a string statement?
        beq.s imp_endnum        ; no. Perhaps end of a numeric statement
        bclr #0,d4              ;if quote flag clear then
        beq.s imp_addlet        ; comma must be taken literally
        bra.s imp_exit          ;else clear quote flag and exit
imp_endnum
        bclr #1,d5              ;if string or numeric
        bne.s imp_exit          ; end of statement else error
*
imp_xp                          ;error in expression
        subq.l #1,a0            ;reset pointer
        addq.w #1,d6            ; and length of remaining input line
        moveq #-17,d1
imp_exit
        move.w d2,0(a6,a1.l)    ;put statement length into place
        subq.l #8,a3            ;point to statement variable
        bsr.s bp_let            ;return statement in variable parameter
        bne imp_err_ret         ;oops
        move.w d6,d0            ;if end of input line
        beq.s imp_skip          ; skip all this
        movea.l a1,a2
        addq.l #2,a2            ;point to to position
imp_mveloop
        addq.l #1,a0            ;point to first character to be moved
        move.b 0(a6,a0.l),0(a6,a2.l) ;move it
        addq.l #1,a2            ;next position
        dbra d0,imp_mveloop     ;until end
imp_skip
        move.w d6,0(a6,a1.l)    ;put length in place
        addq.l #8,a3            ;point to input line parameter
        bsr.s bp_let            ;fill it in with remains of input line
        bne imp_err_ret
        bclr #0,d7              ;a1+ (even)len
        adda.w d7,a1            ; leaving room for integer return
        move.l a1,$58(a6)
imp_x
        move.w d1,0(a6,a1.l)
        moveq #3,d4
        moveq #0,d0
        rts
imp_eof
        moveq #-10,d1
        bra.s imp_x
bp_let
        move.l a1,$58(a6)       ;make temporary top of stack
        movem.l d1/a0-a2,-(a7)
        move.w $120,a2          ;bp.let
        jsr (a2)
        movem.l (a7)+,d1/a0-a2
        rts

        end
